<?php

/**
 * Core SendLane class
 *
 * This class is responsible for all admin + public hooks and overall plugin functionality
 *
 */

class SendLane
{

    protected $loader;
    protected $plugin_name;
    protected $version;

    /**
     * call all deps + hooks
     */
    public function __construct()
    {
        $this->plugin_name = SL_NAME;
        $this->version = SL_VERSION;

        $this->load_dependencies();
        $this->load_admin_hooks();
        $this->load_public_hooks();
    }

    private function load_dependencies()
    {
        require_once SL_PATH . 'inc/sl-loader.php';
        require_once SL_PATH . 'admin/sl-admin.php';
        require_once SL_PATH . 'public/sl-public.php';

        $this->loader = new SendLane_Loader();
    }

    private function load_admin_hooks()
    {
        $sl_admin = new SL_Admin($this->get_plugin_name(), $this->get_plugin_version());

        $this->loader->add_action('admin_enqueue_scripts', $sl_admin, 'sl_enqueue_styles');
        $this->loader->add_action('admin_enqueue_scripts', $sl_admin, 'sl_enqueue_scripts');

        $this->loader->add_action('wp_ajax_save_sendlane_settings', $sl_admin, 'sl_save_admin_settings');
        $this->loader->add_action('wp_ajax_disconnect_sendlane', $sl_admin, 'sl_disconnect_sendlane');

        $this->loader->add_filter('plugin_action_links_' . SL_BASENAME, $sl_admin, 'sl_plugin_action_links', 10, 2); //Adds additional links on Plugin page

        $this->loader->add_filter('woocommerce_settings_tabs_array', $sl_admin, 'sl_register_settings_tab', 50);
        $this->loader->add_action('woocommerce_settings_tabs_sendlane_settings', $sl_admin, 'sl_sendlane_settings_section');

        $this->loader->add_action('init', $sl_admin, 'sl_text_domain'); //Adding language support
        $this->loader->add_filter('sl_remove_empty_carts_hook', $sl_admin, 'delete_empty_carts');
        $this->loader->add_filter('sl_run_sendlaneapi_hook', $sl_admin, 'execute_sendlane_api');
        $this->loader->add_filter('admin_init', $sl_admin, 'show_api_errors');
        $this->loader->add_filter('cron_schedules', $sl_admin, 'sl_additional_cron_intervals'); //Ads a filter to set new interval for Wordpress cron function
    }

    private function load_public_hooks()
    {
        $sl_public = new SL_Public($this->get_plugin_name(), $this->get_plugin_version());

        $this->loader->add_action('wp_enqueue_scripts', $sl_public, 'enqueue_styles');
        $this->loader->add_action('wp_enqueue_scripts', $sl_public, 'enqueue_scripts');

        $this->loader->add_action('init', $sl_public, 'sl_restore_session');
        //$this->loader->add_action('wp_enqueue_scripts', $sl_public, 'sl_additional_scripts_on_checkout');
        $this->loader->add_action('woocommerce_after_checkout_form', $sl_public, 'sl_additional_scripts_on_checkout'); //Adds additional functionality only to Checkout page

        $this->loader->add_action('wp_ajax_nopriv_slsave_data', $sl_public,
            'sl_save_userdata'); //Handles data saving using Ajax after any changes made by the user on the E-mail or Phone field in Checkout form
        $this->loader->add_action('wp_ajax_slsave_data', $sl_public, 'sl_save_userdata'); //Handles data saving using Ajax after any changes made by the user on the E-mail field for Logged in users

        $this->loader->add_action('woocommerce_add_to_cart', $sl_public, 'sl_save_logged_in_user_data',
            200); //Handles data saving if an item is added to shopping cart, 200 = priority set to run the function last after all other functions are finished
        $this->loader->add_action('woocommerce_cart_actions', $sl_public, 'sl_save_logged_in_user_data',
            200); //Handles data updating if a cart is updated. 200 = priority set to run the function last after all other functions are finished
        $this->loader->add_action('woocommerce_cart_item_removed', $sl_public, 'sl_save_logged_in_user_data',
            200); //Handles data updating if an item is removed from cart. 200 = priority set to run the function last after all other functions are finished

        $this->loader->add_action('woocommerce_add_to_cart', $sl_public, 'sl_update_cartdata', 210);
        $this->loader->add_action('woocommerce_cart_actions', $sl_public, 'sl_update_cartdata', 210);
        $this->loader->add_action('woocommerce_cart_item_removed', $sl_public, 'sl_update_cartdata', 210);

        $this->loader->add_action('woocommerce_new_order', $sl_public, 'sl_delete_userdata',
            30); //Hook fired once a new order is created via Checkout process. Order is created as soon as user is taken to payment page. No matter if he pays or not
        $this->loader->add_action('woocommerce_thankyou', $sl_public, 'sl_delete_userdata',
            30); //Hooks into Thank you page to delete a row with a user who completes the checkout (Backup version if first hook does not get triggered after an WooCommerce order gets created)
        ///$this->loader->add_filter( 'woocommerce_checkout_fields', $sl_public, 'restore_input_data', 1); //Restoring previous user input in Checkout form


    }

    public function run()
    {
        $this->loader->run();
    }

    public function get_plugin_name()
    {
        return $this->plugin_name;
    }

    public function get_plugin_version()
    {
        return $this->version;
    }
}
