<?php

/**
 * SL_Public class
 *
 * Handles all the frontend aspects of the plugin
 *
 */

class SL_Public
{

    private $plugin_name;
    private $version;

    public function __construct($plugin_name, $version)
    {
        global $wpdb;
        $this->plugin_name = $plugin_name;
        $this->version = $version;
    }

    public function enqueue_styles()
    {
        wp_enqueue_style($this->plugin_name, SL_URL . 'public/css/sl-frontend.css', [], $this->version, 'all');
    }

    public function enqueue_scripts()
    {
    }

    public function sl_additional_scripts_on_checkout()
    {
        wp_enqueue_script($this->plugin_name, SL_URL . 'public/js/sl-frontend.js', ['jquery'], $this->version, false);
        wp_localize_script($this->plugin_name, 'ajaxLink', ['ajaxurl' => admin_url('admin-ajax.php')]);
    }

    /**
     * Restore cart session when cart link opened via email
     */

    public function sl_restore_session()
    {

        if (isset($_GET['sl_session'])) {
            $ses = esc_sql($_GET['sl_session']);

            global $wpdb;
            $table_name = $wpdb->prefix . SL_TABLE;
            $cart = $wpdb->get_var(
                $wpdb->prepare("SELECT `cart_contents` FROM $table_name WHERE `session_id`=%s", $ses)
            );

            $cart_data = unserialize($cart);

            if (is_array($cart_data) && count($cart_data) >= 1) {
                WC()->cart->empty_cart();

                foreach ($cart_data as $product) {

                    // Validate Product data
                    $product_id = isset($product['product_id']) ? (int) $product['product_id'] : 0;
                    $quantity = isset($product['quantity']) ? (int) $product['quantity'] : 1;
                    $variation_id = isset($product['product_variation_id']) ? (int) $product['product_variation_id'] : 0;
                    $variation = isset($product['variation']) ? $product['variation'] : [];

                    WC()->cart->add_to_cart($product_id, $quantity, $variation_id, $variation);
                }
                WC()->cart->calculate_totals();
            }
        }
    }

    /**
     *
     * Data Functions
     * Save user data from checkout input
     *
     */

    function sl_save_userdata()
    {
        //First check if data is being sent and that it is the data we want
        if (isset($_POST["sl_email"])) {
            global $wpdb;
            $table_name = $wpdb->prefix . SL_TABLE; // do not forget about tables prefix

            //Retrieving cart array consisting of currency, cart toal, time, session id and products and their quantities
            $cart_data = $this->sl_read_cart();
            $cart_total = $cart_data['cart_total'];
            $cart_currency = $cart_data['cart_currency'];
            $current_time = $cart_data['current_time'];
            $session_id = $cart_data['session_id'];
            $product_array = $cart_data['product_array'];
            $sl_session_id = WC()->session->get('sl_session_id');

            //In case if the cart has no items in it, we need to delete the abandoned cart
            if (empty($product_array)) {
                $this->sl_clear_cart_data();
                return;
            }

            //Checking if we have values coming from the input fields
            (isset($_POST['sl_name'])) ? $name = $_POST['sl_name'] : $name = ''; //If/Else shorthand (condition) ? True : False
            (isset($_POST['sl_surname'])) ? $surname = $_POST['sl_surname'] : $surname = '';
            (isset($_POST['sl_phone'])) ? $phone = $_POST['sl_phone'] : $phone = '';
            (isset($_POST['sl_country'])) ? $country = $_POST['sl_country'] : $country = '';
            (isset($_POST['sl_city']) && $_POST['sl_city'] != '') ? $city = ", " . $_POST['sl_city'] : $city = '';
            (isset($_POST['sl_billing_company'])) ? $company = $_POST['sl_billing_company'] : $company = '';
            (isset($_POST['sl_billing_address_1'])) ? $address_1 = $_POST['sl_billing_address_1'] : $address_1 = '';
            (isset($_POST['sl_billing_address_2'])) ? $address_2 = $_POST['sl_billing_address_2'] : $address_2 = '';
            (isset($_POST['sl_billing_state'])) ? $state = $_POST['sl_billing_state'] : $state = '';
            (isset($_POST['sl_billing_postcode'])) ? $postcode = $_POST['sl_billing_postcode'] : $postcode = '';
            (isset($_POST['sl_shipping_first_name'])) ? $shipping_name = $_POST['sl_shipping_first_name'] : $shipping_name = '';
            (isset($_POST['sl_shipping_last_name'])) ? $shipping_surname = $_POST['sl_shipping_last_name'] : $shipping_surname = '';
            (isset($_POST['sl_shipping_company'])) ? $shipping_company = $_POST['sl_shipping_company'] : $shipping_company = '';
            (isset($_POST['sl_shipping_country'])) ? $shipping_country = $_POST['sl_shipping_country'] : $shipping_country = '';
            (isset($_POST['sl_shipping_address_1'])) ? $shipping_address_1 = $_POST['sl_shipping_address_1'] : $shipping_address_1 = '';
            (isset($_POST['sl_shipping_address_2'])) ? $shipping_address_2 = $_POST['sl_shipping_address_2'] : $shipping_address_2 = '';
            (isset($_POST['sl_shipping_city'])) ? $shipping_city = $_POST['sl_shipping_city'] : $shipping_city = '';
            (isset($_POST['sl_shipping_state'])) ? $shipping_state = $_POST['sl_shipping_state'] : $shipping_state = '';
            (isset($_POST['sl_shipping_postcode'])) ? $shipping_postcode = $_POST['sl_shipping_postcode'] : $shipping_postcode = '';
            (isset($_POST['sl_order_comments'])) ? $comments = $_POST['sl_order_comments'] : $comments = '';
            (isset($_POST['sl_create_account'])) ? $create_account = $_POST['sl_create_account'] : $create_account = '';
            (isset($_POST['sl_ship_elsewhere'])) ? $ship_elsewhere = $_POST['sl_ship_elsewhere'] : $ship_elsewhere = '';

            $other_fields = [
                'sl_billing_company' => $company,
                'sl_billing_address_1' => $address_1,
                'sl_billing_address_2' => $address_2,
                'sl_billing_state' => $state,
                'sl_billing_postcode' => $postcode,
                'sl_shipping_first_name' => $shipping_name,
                'sl_shipping_last_name' => $shipping_surname,
                'sl_shipping_company' => $shipping_company,
                'sl_shipping_country' => $shipping_country,
                'sl_shipping_address_1' => $shipping_address_1,
                'sl_shipping_address_2' => $shipping_address_2,
                'sl_shipping_city' => $shipping_city,
                'sl_shipping_state' => $shipping_state,
                'sl_shipping_postcode' => $shipping_postcode,
                'sl_order_comments' => $comments,
                'sl_create_account' => $create_account,
                'sl_ship_elsewhere' => $ship_elsewhere
            ];

            $location = $country . $city;

            $current_session_exist_in_db = $this->sl_current_session_exist_in_db($sl_session_id);
            //If we have already inserted the Users session ID in Session variable and it is not NULL and Current session ID exists in Database we update the abandoned cart row
            if ($current_session_exist_in_db && $sl_session_id !== null) {

                //Updating row in the Database where users Session id = same as prevously saved in Session
                $wpdb->prepare(
                    '%s',
                    $wpdb->update(
                        $table_name,
                        [
                            'name' => sanitize_text_field($name),
                            'surname' => sanitize_text_field($surname),
                            'email' => sanitize_email($_POST['sl_email']),
                            'phone' => filter_var($phone, FILTER_SANITIZE_NUMBER_INT),
                            'location' => sanitize_text_field($location),
                            'cart_contents' => serialize($product_array),
                            'cart_total' => sanitize_text_field($cart_total),
                            'currency' => sanitize_text_field($cart_currency),
                            'time' => sanitize_text_field($current_time),
                            'other_fields' => sanitize_text_field(serialize($other_fields))
                        ],
                        ['session_id' => $sl_session_id],
                        ['%s', '%s', '%s', '%s', '%s', '%s', '%0.2f', '%s', '%s', '%s'],
                        ['%s']
                    )
                );
            } else {

                //Inserting row into Database
                $wpdb->query(
                    $wpdb->prepare(
                        "INSERT INTO " . $table_name . "
						( name, surname, email, phone, location, cart_contents, cart_total, currency, time, session_id, other_fields )
						VALUES ( %s, %s, %s, %s, %s, %s, %0.2f, %s, %s, %s, %s)",
                        [
                            sanitize_text_field($name),
                            sanitize_text_field($surname),
                            sanitize_email($_POST['sl_email']),
                            filter_var($phone, FILTER_SANITIZE_NUMBER_INT),
                            sanitize_text_field($location),
                            serialize($product_array),
                            sanitize_text_field($cart_total),
                            sanitize_text_field($cart_currency),
                            sanitize_text_field($current_time),
                            sanitize_text_field($session_id),
                            sanitize_text_field(serialize($other_fields))
                        ]
                    )
                );

                //Storing session_id in WooCommerce session
                WC()->session->set('sl_session_id', $session_id);
                $this->slincrease_captured_abandoned_cart_count(); //Updating total count of captured abandoned carts
            }

            die();
        }
    }

    /**
     * Function automatically saves a cart if a logged in user adds something to his shopping cart, removes something or updates his cart
     * If we are updating the cart, we do not change users data, but just the Cart data since user can only change his data in the Checkout form
     *
     */
    function sl_save_logged_in_user_data()
    {
        if (is_user_logged_in()) { //If a user is logged in
            global $wpdb;
            $table_name = $wpdb->prefix . SL_TABLE; // do not forget about tables prefix

            //Retrieving cart array consisting of currency, cart toal, time, session id and products and their quantities
            $cart_data = $this->sl_read_cart();
            $cart_total = $cart_data['cart_total'];
            $cart_currency = $cart_data['cart_currency'];
            $current_time = $cart_data['current_time'];
            $session_id = $cart_data['session_id'];
            $product_array = $cart_data['product_array'];
            $sl_session_id = WC()->session->get('sl_session_id');

            //In case if the user updates the cart and takes out all items from the cart
            if (empty($product_array)) {
                $this->sl_clear_cart_data();
                return;
            }

            $abandoned_cart = '';

            //If we haven't set sl_session_id, then need to check in the database if the current user has got an abandoned cart already
            if ($sl_session_id === null) {
                $main_table = $wpdb->prefix . SL_TABLE;
                $abandoned_cart = $wpdb->get_row(
                    $wpdb->prepare(
                        "SELECT session_id FROM " . $main_table . "
					WHERE session_id = %d",
                        get_current_user_id()
                    )
                );
            }

            $current_session_exist_in_db = $this->sl_current_session_exist_in_db($sl_session_id);
            //If the current user has got an abandoned cart already or if we have already inserted the Users session ID in Session variable and it is not NULL and already inserted the Users session ID in Session variable we update the abandoned cart row
            if ($current_session_exist_in_db && (!empty($abandoned_cart) || $sl_session_id !== null)) {

                //If the user has got an abandoned cart previously, we set session ID back
                if (!empty($abandoned_cart)) {
                    $session_id = $abandoned_cart->session_id;
                    //Storing session_id in WooCommerce session
                    WC()->session->set('sl_session_id', $session_id);
                } else {
                    $session_id = $sl_session_id;
                }

                //Updating row in the Database where users Session id = same as prevously saved in Session
                //Updating only Cart related data since the user can change his data only in the Checkout form
                $wpdb->prepare(
                    '%s',
                    $wpdb->update(
                        $table_name,
                        [
                            'cart_contents' => serialize($product_array),
                            'cart_total' => sanitize_text_field($cart_total),
                            'currency' => sanitize_text_field($cart_currency),
                            'time' => sanitize_text_field($current_time)
                        ],
                        ['session_id' => $session_id],
                        ['%s', '%0.2f', '%s', '%s'],
                        ['%s']
                    )
                );
            } else {

                //Looking if a user has previously made an order
                //If not, using default WordPress assigned data
                //Handling users name
                $current_user = wp_get_current_user(); //Retrieving users data
                if ($current_user->billing_first_name) {
                    $name = $current_user->billing_first_name;
                } else {
                    $name = $current_user->user_firstname; //Users name
                }

                //Handling users surname
                if ($current_user->billing_last_name) {
                    $surname = $current_user->billing_last_name;
                } else {
                    $surname = $current_user->user_lastname;
                }

                //Handling users email address
                if ($current_user->billing_email) {
                    $email = $current_user->billing_email;
                } else {
                    $email = $current_user->user_email;
                }

                //Handling users phone
                $phone = $current_user->billing_phone;

                //Handling users address
                if ($current_user->billing_country) {
                    $country = $current_user->billing_country;
                    if ($current_user->billing_state) { //checking if the state was entered
                        $city = ", " . $current_user->billing_state;
                    } else {
                        $city = '';
                    }
                    $location = $country . $city;
                } else {
                    $location = WC_Geolocation::geolocate_ip(); //Getting users country from his IP address
                    $location = $location['country'];
                }

                //Inserting row into Database
                $wpdb->query(
                    $wpdb->prepare(
                        "INSERT INTO " . $table_name . "
						( name, surname, email, phone, location, cart_contents, cart_total, currency, time, session_id)
						VALUES ( %s, %s, %s, %s, %s, %s, %0.2f, %s, %s, %s)",
                        [
                            sanitize_text_field($name),
                            sanitize_text_field($surname),
                            sanitize_email($email),
                            filter_var($phone, FILTER_SANITIZE_NUMBER_INT),
                            sanitize_text_field($location),
                            serialize($product_array),
                            sanitize_text_field($cart_total),
                            sanitize_text_field($cart_currency),
                            sanitize_text_field($current_time),
                            sanitize_text_field($session_id)
                        ]
                    )
                );
                //Storing session_id in WooCommerce session
                WC()->session->set('sl_session_id', $session_id);

                $this->slincrease_captured_abandoned_cart_count(); //Increasing total count of captured abandoned carts
            }
        }
    }

    /**
     * Function updates cart data for users who are not signed in
     */
    function sl_update_cartdata()
    {
        if (!is_user_logged_in()) { //If a user is not logged in

            $sl_session_id = WC()->session->get('sl_session_id');
            if ($sl_session_id !== null) {

                global $wpdb;
                $table_name = $wpdb->prefix . SL_TABLE;
                $cart_data = $this->sl_read_cart();
                $product_array = $cart_data['product_array'];
                $cart_total = $cart_data['cart_total'];
                $cart_currency = $cart_data['cart_currency'];
                $current_time = $cart_data['current_time'];

                //In case if the cart has no items in it, we need to delete the abandoned cart
                if (empty($product_array)) {
                    $this->sl_clear_cart_data();
                    return;
                }

                //Updating row in the Database where users Session id = same as prevously saved in Session
                $wpdb->prepare(
                    '%s',
                    $wpdb->update(
                        $table_name,
                        [
                            'cart_contents' => serialize($product_array),
                            'cart_total' => sanitize_text_field($cart_total),
                            'currency' => sanitize_text_field($cart_currency),
                            'time' => sanitize_text_field($current_time)
                        ],
                        ['session_id' => $sl_session_id],
                        ['%s', '%0.2f', '%s', '%s'],
                        ['%s']
                    )
                );
            }
        }
    }

    /**
     * Function checks if current user session ID also exists in the database
     */
    function sl_current_session_exist_in_db($sl_session_id)
    {
        //If we have saved the abandoned cart in session variable
        if ($sl_session_id !== null) {
            global $wpdb;
            $main_table = $wpdb->prefix . SL_TABLE;

            //Checking if we have this abandoned cart in our database already
            return $result = $wpdb->get_var($wpdb->prepare(
                "SELECT session_id
				FROM " . $main_table . "
				WHERE session_id = %s",
                $sl_session_id
            ));
        } else {
            return false;
        }
    }

    /**
     * Function to clear cart data from row
     */
    function sl_clear_cart_data()
    {

        global $wpdb;
        $table_name = $wpdb->prefix . SL_TABLE; // do not forget about tables prefix

        //If a new Order is added from the WooCommerce admin panel, we must check if WooCommerce session is set. Otherwise we would get a Fatal error.
        if (isset(WC()->session)) {

            $sl_session_id = WC()->session->get('sl_session_id');
            if (isset($sl_session_id)) {

                $cart_data = $this->sl_read_cart();
                $cart_currency = $cart_data['cart_currency'];
                $current_time = $cart_data['current_time'];

                //Cleaning Cart data
                $wpdb->prepare(
                    '%s',
                    $wpdb->update(
                        $table_name,
                        [
                            'cart_contents' => '',
                            'cart_total' => 0,
                            'currency' => sanitize_text_field($cart_currency),
                            'time' => sanitize_text_field($current_time)
                        ],
                        ['session_id' => $sl_session_id],
                        ['%s', '%s'],
                        ['%s']
                    )
                );
            }
        }
    }

    /**
     * Function in order to delete row from table if the user completes the checkout
     *
     * @since    1.3
     */
    function sl_delete_userdata()
    {
        global $wpdb;
        $table_name = $wpdb->prefix . SL_TABLE; // do not forget about tables prefix

        //If a new Order is added from the WooCommerce admin panel, we must check if WooCommerce session is set. Otherwise we would get a Fatal error.
        if (isset(WC()->session)) {

            $sl_session_id = WC()->session->get('sl_session_id');
            if (isset($sl_session_id)) {

                //Deleting row from database
                $wpdb->query(
                    $wpdb->prepare(
                        "DELETE FROM " . $table_name . "
						 WHERE session_id = %s",
                        sanitize_key($sl_session_id)
                    )
                );
                $this->sldecrease_captured_abandoned_cart_count($count = false); //Decreasing total count of captured abandoned carts
            }

            $this->slunset_session_id();
        }
    }

    /**
     *
     * Function for reading the cart
     *
     */
    function sl_read_cart()
    {
        global $woocommerce;

        //Retrieving cart total value and currency
        $cart_total = WC()->cart->total;
        $cart_currency = get_woocommerce_currency();
        $current_time = current_time('mysql', false); //Retrieving current time

        //Retrieving customer ID from WooCommerce sessions variable in order to use it as a session_id value
        $session_id = WC()->session->get_customer_id();

        //Retrieving cart
        $products = $woocommerce->cart->cart_contents;
        $product_array = [];

        foreach ($products as $product => $values) {
            $item = wc_get_product($values['data']->get_id());

            $product_title = $item->get_title();
            $product_quantity = $values['quantity'];
            $product_variation_price = $values['line_total'];

            // Handling product variations
            if ($values['variation_id']) { //If user has chosen a variation
                $single_variation = new WC_Product_Variation($values['variation_id']);

                //Handling variable product title output with attributes
                $product_attributes = $this->sl_attribute_slug_to_title($single_variation->get_variation_attributes());
                $product_variation_id = $values['variation_id'];
            } else {
                $product_attributes = false;
                $product_variation_id = '';
            }

            //Inserting Product title, Variation and Quantity into array
            $product_array[] = [
                'product_title' => $product_title . $product_attributes,
                'quantity' => $product_quantity,
                'product_id' => $values['product_id'],
                'product_variation_id' => $product_variation_id,
                'product_variation_price' => $product_variation_price
            ];
        }

        return $results_array = ['cart_total' => $cart_total, 'cart_currency' => $cart_currency, 'current_time' => $current_time, 'session_id' => $session_id, 'product_array' => $product_array];
    }


    /***
     *
     * Utility Functions
     *
     */
    public function sl_attribute_slug_to_title($product_variations)
    {
        global $woocommerce;
        $attribute_array = [];

        if ($product_variations) {

            foreach ($product_variations as $product_variation_key => $product_variation_name) {

                $value = '';
                if (taxonomy_exists(esc_attr(str_replace('attribute_', '', $product_variation_key)))) {
                    $term = get_term_by('slug', $product_variation_name, esc_attr(str_replace('attribute_', '', $product_variation_key)));
                    if (!is_wp_error($term) && !empty($term->name)) {
                        $value = $term->name;
                        if (!empty($value)) {
                            $attribute_array[] = $value;
                        }
                    }
                } else {
                    $value = apply_filters('woocommerce_variation_option_name', $product_variation_name);
                    if (!empty($value)) {
                        $attribute_array[] = $value;
                    }
                }
            }

            //Generating attribute output
            $total_variations = count($attribute_array);
            $increment = 0;
            $product_attribute = '';
            foreach ($attribute_array as $attribute) {
                if ($increment === 0 && $increment != $total_variations - 1) { //If this is first variation and we have multiple variations
                    $colon = ': ';
                    $comma = ', ';
                } elseif ($increment === 0 && $increment === $total_variations - 1) { //If we have only one variation
                    $colon = ': ';
                    $comma = false;
                } elseif ($increment === $total_variations - 1) { //If this is the last variation
                    $comma = '';
                    $colon = false;
                } else {
                    $comma = ', ';
                    $colon = false;
                }
                $product_attribute .= $colon . $attribute . $comma;
                $increment++;
            }
            return $product_attribute;
        } else {
            return;
        }
    }

    function slunset_session_id()
    {
        //Removing stored ID value from WooCommerce Session
        WC()->session->__unset('sl_session_id');
    }

    /**
     * Function saves and updates total count of captured abandoned carts
     */
    function slincrease_captured_abandoned_cart_count()
    {
        $previously_captured_abandoned_cart_count = get_option('captured_abandoned_cart_count');
        update_option('captured_abandoned_cart_count', $previously_captured_abandoned_cart_count + 1); //Updating the count by one abandoned cart
    }

    /**
     * Function decreases the total count of captured abandoned carts
     */
    function sldecrease_captured_abandoned_cart_count($count)
    {
        if (!$count) {
            $count = 1;
        }
        $previously_captured_abandoned_cart_count = get_option('captured_abandoned_cart_count');
        update_option('captured_abandoned_cart_count', $previously_captured_abandoned_cart_count - $count); //Decreasing the count by one abandoned cart
    }
}
